package com.maddyhome.idea.color.config;

/*
 * Color Browser - Color browser plugin for IDEA
 * Copyright (C) 2006 Rick Maddy. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import com.intellij.openapi.diagnostic.Logger;
import com.maddyhome.idea.color.util.XmlUtil;
import org.jetbrains.annotations.NotNull;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.awt.Color;
import java.util.Map;
import java.util.TreeMap;

public class Configuration
{
    @NotNull
    public static synchronized Configuration getInstance()
    {
        if (instance == null)
        {
            instance = new Configuration();
        }

        return instance;
    }

    @NotNull
    public Map<Integer, String> getSVGColors()
    {
        return svgColors;
    }

    @NotNull
    public Map<String, Integer> getSVGNames()
    {
        return svgNames;
    }

    @NotNull
    public Map<Integer, String> getJavaColors()
    {
        return javaColors;
    }

    @NotNull
    public Map<String, Integer> getJavaNames()
    {
        return javaNames;
    }

    public String findSVGName(Color color)
    {
        int rgb = color.getRGB() & 0xffffff;

        return svgColors.get(rgb);
    }

    public Color findSVGColor(String name)
    {
        Integer code = svgNames.get(name);
        if (code != null)
        {
            return new Color(code);
        }

        return null;
    }

    public String findJavaName(Color color)
    {
        int rgb = color.getRGB() & 0xffffff;

        return javaColors.get(rgb);
    }

    public Color findJavaColor(String name)
    {
        Integer code = javaNames.get(name);
        if (code != null)
        {
            return new Color(code);
        }

        return null;
    }

    private Configuration()
    {
        loadColors();
    }

    private void loadColors()
    {
        try
        {
            svgColors = new TreeMap<Integer, String>();
            svgNames = new TreeMap<String, Integer>();

            Document doc = XmlUtil.loadDocument("/config/colors.xml");
            Element root = doc.getDocumentElement();
            NodeList ents = root.getElementsByTagName("color");
            for (int i = 0; i < ents.getLength(); i++)
            {
                Element ent = (Element)ents.item(i);

                svgColors.put(Integer.valueOf(ent.getAttribute("code"), 16), ent.getAttribute("name"));
                svgNames.put(ent.getAttribute("name"), Integer.valueOf(ent.getAttribute("code"), 16));
            }
        }
        catch (Exception e)
        {
            logger.error("Unable to load svgColors", e);
        }

        Object[] colors = new Object[] {
            Color.black, "black",
            Color.blue, "blue",
            Color.cyan, "cyan",
            Color.darkGray, "darkgray",
            Color.gray, "gray",
            Color.green, "green",
            Color.lightGray, "lightgray",
            Color.magenta, "magenta",
            Color.orange, "orange",
            Color.pink, "pink",
            Color.red, "red",
            Color.white, "white",
            Color.yellow, "yellow",
            Color.BLACK, "BLACK",
            Color.BLUE, "BLUE",
            Color.CYAN, "CYAN",
            Color.DARK_GRAY, "DARK_GRAY",
            Color.GRAY, "GRAY",
            Color.GREEN, "GREEN",
            Color.LIGHT_GRAY, "LIGHT_GRAY",
            Color.MAGENTA, "MAGENTA",
            Color.ORANGE, "ORANGE",
            Color.PINK, "PINK",
            Color.RED, "RED",
            Color.WHITE, "WHITE",
            Color.YELLOW, "YELLOW"
        };

        javaColors = new TreeMap<Integer, String>();
        javaNames = new TreeMap<String, Integer>();
        for (int i = 0; i < colors.length; i += 2)
        {
            int col = ((Color)colors[i]).getRGB() & 0xffffff;
            javaColors.put(col, (String)colors[i + 1]);
            javaNames.put((String)colors[i + 1], col);
        }
    }

    private Map<Integer, String> svgColors;
    private Map<String, Integer> svgNames;
    private Map<Integer, String> javaColors;
    private Map<String, Integer> javaNames;

    private static Configuration instance = null;

    private static Logger logger = Logger.getInstance(Configuration.class.getName());
}